/*
    This file is part of the Okteta Core library, made within the KDE community.

    SPDX-FileCopyrightText: 2007 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef BOOKMARKLISTTEST_HPP
#define BOOKMARKLISTTEST_HPP

// Qt
#include <QObject>

class BookmarkListTest : public QObject
{
    Q_OBJECT

private Q_SLOTS: // test functions
    void testSimpleConstructor();
    void testAddRemoveBookmark();
    void testContains();
//     void testPreviousFrom();
//     void testNextFrom();
    void testAdjustToSwapped();
    void testAdjustToReplaced();
};

#endif
