<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="oktetapartreadonly" version="3" translationDomain="oktetapart">

<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_save_as" group="save_merge" />
    <Action name="export" group="save_merge" />
    <Action name="file_print" group="print_merge" />
  </Menu>
  <Menu name="edit"><text>&amp;Edit</text>
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="copy_as" group="edit_paste_merge" />
    <Separator group="edit_paste_merge" />
    <Action name="edit_select_all" group="edit_select_merge" />
    <Action name="edit_deselect" group="edit_select_merge" />
    <Action name="set_verticalSelect" group="edit_select_merge" />
    <Action name="edit_select" group="edit_select_merge" />
    <Separator group="edit_select_merge" />
    <Action name="edit_find" group="edit_find_merge" />
    <Action name="edit_find_next" group="edit_find_merge" />
    <Action name="edit_find_prev" group="edit_find_merge" />
    <Action name="edit_replace" group="edit_find_merge" />
    <Separator group="edit_find_merge" />
    <Action name="goto_offset" group="edit_find_merge" />
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="view_zoom_in" group="view_operations" />
    <Action name="view_zoom_out" group="view_operations" />
    <Separator group="view_operations" />
    <Action name="view_lineoffset" group="view_operations" />
    <Action name="view_offsetcoding" />
    <Separator group="view_operations" />
    <Action name="togglecolumns" group="view_operations" />
    <Action name="view_valuecoding" group="view_operations" />
    <Action name="view_charencoding" group="view_operations" />
    <Action name="view_showsnonprinting" group="view_operations" />
    <Separator group="view_operations" />
    <Action name="view_bytesperline" group="view_operations" />
    <Action name="view_bytespergroup" group="view_operations" />
    <Action name="resizestyle" group="view_operations" />
    <Action name="viewmode" group="view_operations" />
    <Separator/>
    <Action name="view_profile" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="edit_find" />
</ToolBar>

<Menu name="viewContextMenu">
  <Action name="edit_copy"/>
  <Action name="copy_as"/>
  <Separator/>
  <Action name="edit_select_all"/>
  <Action name="edit_deselect"/>
  <Action name="edit_select"/>
</Menu>

</gui>
