/*
    This file is part of the Okteta Gui library, made within the KDE community.

    SPDX-FileCopyrightText: 2009 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef OKTETA_ABSTRACTCOLUMNSTYLIST_P_HPP
#define OKTETA_ABSTRACTCOLUMNSTYLIST_P_HPP

// lib
#include "abstractcolumnstylist.hpp"

namespace Okteta {

class AbstractColumnStylistPrivate
{
public:
    AbstractColumnStylistPrivate();
    AbstractColumnStylistPrivate(const AbstractColumnStylistPrivate&) = delete;

    virtual ~AbstractColumnStylistPrivate();

    AbstractColumnStylistPrivate& operator=(const AbstractColumnStylistPrivate&) = delete;

public:

private:
};

inline AbstractColumnStylistPrivate::AbstractColumnStylistPrivate() = default;
inline AbstractColumnStylistPrivate::~AbstractColumnStylistPrivate() = default;

}

#endif
