/*
    This file is part of the Kasten Framework, made within the KDE community.

    SPDX-FileCopyrightText: 2008 Friedrich W. H. Kossebau <kossebau@kde.org>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#include "abstractselectionview.hpp"

namespace Kasten {

class AbstractSelectionViewPrivate
{
public:
};


AbstractSelectionView::AbstractSelectionView() = default;

AbstractSelectionView::~AbstractSelectionView() = default;

}

#include "moc_abstractselectionview.cpp"
